#define MOZ_SOURCE_STAMP cb27eacbe04abc5f91a0379c23617715aab432ec
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/cb27eacbe04abc5f91a0379c23617715aab432ec
