/* THIS FILE IS AUTOGENERATED FROM MozCellBroadcastMessage.webidl BY Codegen.py - DO NOT EDIT */

#ifndef mozilla_dom_MozCellBroadcastMessageBinding_h
#define mozilla_dom_MozCellBroadcastMessageBinding_h

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

class CellBroadcastEtwsInfo;
struct CellBroadcastEtwsInfoAtoms;
class CellBroadcastMessage;
struct CellBroadcastMessageAtoms;
struct NativePropertyHooks;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

enum class CellBroadcastGsmGeographicalScope : uint32_t {
  Cell_immediate,
  Plmn,
  Location_area,
  Cell,
  EndGuard_
};

namespace CellBroadcastGsmGeographicalScopeValues {
extern const EnumEntry strings[5];
} // namespace CellBroadcastGsmGeographicalScopeValues

bool
ToJSValue(JSContext* aCx, CellBroadcastGsmGeographicalScope aArgument, JS::MutableHandle<JS::Value> aValue);


enum class CellBroadcastMessageClass : uint32_t {
  Class_0,
  Class_1,
  Class_2,
  Class_3,
  User_1,
  User_2,
  Normal,
  EndGuard_
};

namespace CellBroadcastMessageClassValues {
extern const EnumEntry strings[8];
} // namespace CellBroadcastMessageClassValues

bool
ToJSValue(JSContext* aCx, CellBroadcastMessageClass aArgument, JS::MutableHandle<JS::Value> aValue);


enum class CellBroadcastEtwsWarningType : uint32_t {
  Earthquake,
  Tsunami,
  Earthquake_tsunami,
  Test,
  Other,
  EndGuard_
};

namespace CellBroadcastEtwsWarningTypeValues {
extern const EnumEntry strings[6];
} // namespace CellBroadcastEtwsWarningTypeValues

bool
ToJSValue(JSContext* aCx, CellBroadcastEtwsWarningType aArgument, JS::MutableHandle<JS::Value> aValue);


namespace MozCellBroadcastEtwsInfoBinding {

  typedef mozilla::dom::CellBroadcastEtwsInfo NativeType;

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];

  JSObject*
  DefineDOMInterface(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Handle<jsid> id, bool aDefineOnGlobal);

  bool
  ConstructorEnabled(JSContext* aCx, JS::Handle<JSObject*> aObj);

  const JSClass*
  GetJSClass();

  bool
  Wrap(JSContext* aCx, mozilla::dom::CellBroadcastEtwsInfo* aObject, nsWrapperCache* aCache, JS::Handle<JSObject*> aGivenProto, JS::MutableHandle<JSObject*> aReflector);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, T* aObject, JS::Handle<JSObject*> aGivenProto)
  {
    JS::Rooted<JSObject*> reflector(aCx);
    return Wrap(aCx, aObject, aObject, aGivenProto, &reflector) ? reflector.get() : nullptr;
  }

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, ProtoAndIfaceCache& aProtoAndIfaceCache, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObjectHandle(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

  JS::Handle<JSObject*>
  GetConstructorObjectHandle(JSContext* aCx, JS::Handle<JSObject*> aGlobal, bool aDefineOnGlobal = true);

  JSObject*
  GetConstructorObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

} // namespace MozCellBroadcastEtwsInfoBinding



namespace MozCellBroadcastMessageBinding {

  typedef mozilla::dom::CellBroadcastMessage NativeType;

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];

  JSObject*
  DefineDOMInterface(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Handle<jsid> id, bool aDefineOnGlobal);

  bool
  ConstructorEnabled(JSContext* aCx, JS::Handle<JSObject*> aObj);

  const JSClass*
  GetJSClass();

  bool
  Wrap(JSContext* aCx, mozilla::dom::CellBroadcastMessage* aObject, nsWrapperCache* aCache, JS::Handle<JSObject*> aGivenProto, JS::MutableHandle<JSObject*> aReflector);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, T* aObject, JS::Handle<JSObject*> aGivenProto)
  {
    JS::Rooted<JSObject*> reflector(aCx);
    return Wrap(aCx, aObject, aObject, aGivenProto, &reflector) ? reflector.get() : nullptr;
  }

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, ProtoAndIfaceCache& aProtoAndIfaceCache, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObjectHandle(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

  JS::Handle<JSObject*>
  GetConstructorObjectHandle(JSContext* aCx, JS::Handle<JSObject*> aGlobal, bool aDefineOnGlobal = true);

  JSObject*
  GetConstructorObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

} // namespace MozCellBroadcastMessageBinding



} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_MozCellBroadcastMessageBinding_h
